<?php

namespace Drupal\twig_ui\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Checkboxes;
use Drupal\twig_ui\TemplateManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The settings form for the Twig UI module.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The Twig UI Template Manager service.
   *
   * @var \Drupal\twig_ui\TemplateManager
   */
  protected $templateManager;

  /**
   * Constructs a SettingsForm object.
   *
   * @param \Drupal\twig_ui\TemplateManager $template_manager
   *   The Twig UI Template Manager service.
   */
  public function __construct(TemplateManager $template_manager) {
    $this->templateManager = $template_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('twig_ui.template_manager')
    );
  }

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'twig_ui.settings';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'twig_ui_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $options = [];
    $options['_default'] = 'Default system theme';
    $list = $this->templateManager->getActiveThemes();
    foreach ($list as $theme_name => $theme) {
      $options[$theme_name] = $theme->info['name'];
    }

    $form['default_selected_themes'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Default theme(s)'),
      '#description' => $this->t('The themes selected here will be pre-selected as the default theme values for new Twig UI templates.'),
      '#options' => $options,
      '#default_value' => $config->get('default_selected_themes') ?: [],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Run 'themes' values through Checkboxes::getCheckedCheckboxes()
    // to remove unchecked values.
    if (!empty($form_state->getValue('default_selected_themes'))) {
      $form_state->setValue('default_selected_themes', Checkboxes::getCheckedCheckboxes($form_state->getValue('default_selected_themes')));
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable(static::SETTINGS)
      ->set('default_selected_themes', $form_state->getValue('default_selected_themes'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
